@@ -25,8 +25,6 @@
 #include "analitzaexport.h"
 #include "object.h"
 
-class QDomElement;
-
 namespace Analitza
 {
 class Ci;
@@ -34,10 +32,14 @@ class Container;
 class Cn;
 
 /**
- *	This class represents an expression.
- *	Expression let to convert it to string, MathML and make some little queries to it, without calculating anything.
+ * \class Expression
+ * 
+ * \ingroup AnalitzaModule
  *
- *	@author Aleix Pol <aleixpol@kde.org>  
+ * \brief Represents a mathematical expression.
+ *
+ * Expression let to convert it to string, MathML and make some little queries 
+ * to it, without calculating anything.
  */
 
 class ANALITZA_EXPORT Expression
@@ -136,6 +138,11 @@ class ANALITZA_EXPORT Expression
 		 */
 		Expression elementAt(int position) const;
 		
+		/**
+		 *	sets an expression value @p value to a @p position
+		 */
+		void setElementAt(int position, const Analitza::Expression& exp);
+		
 		/**
 		 *	Returns the tree associated to this object.
 		 */
