@@ -30,6 +30,7 @@
 #include "expressiontypechecker.h"
 #include "customobject.h"
 #include "localize.h"
+#include "matrix.h"
 
 using namespace std;
 using namespace Analitza;
@@ -37,96 +38,82 @@ using namespace Analitza;
 Cn* Operations::reduceRealReal(enum Operator::OperatorType op, Cn *oper, const Cn *oper1, QString** correct)
 {
 	int residu;
-	double a=oper->value(), b=oper1->value();
-	Cn::ValueFormat format=Cn::ValueFormat(oper->format() | oper1->format());
+	const double a=oper->value(), b=oper1->value();
 	
 	switch(op) {
 		case Operator::plus:
-			a += b;
+			oper->setValue(a+b);
 			break;
 		case Operator::times:
-			a *= b;
+			oper->setValue(a*b);
 			break;
 		case Operator::divide:
 			if(KDE_ISLIKELY(b!=0.))
-				a /= b;
+				oper->setValue(a / b);
 			else
 				*correct=new QString(i18n("Cannot divide by 0."));
 			break;
 		case Operator::minus:
-			a -= b;
+			oper->setValue(a - b);
 			break;
 		case Operator::power:
-			a = pow(a, b);
+			oper->setValue(b==2 ? a*a
+								: pow(a, b));
 			break;
 		case Operator::rem:
-			format=Cn::Integer;
 			if(KDE_ISLIKELY(floor(b)!=0.))
-				a = remainder(a, b);
+				oper->setValue(int(remainder(a, b)));
 			else
 				*correct=new QString(i18n("Cannot calculate the remainder on 0."));
 			break;
 		case Operator::quotient:
-			a = floor(a / b);
-			format=Cn::Integer;
+			oper->setValue(int(floor(a / b)));
 			break;
-		case Operator::factorof:
-			if(KDE_ISLIKELY(floor(b)!=0.))
-				a = ((int(floor(a)) % int(floor(b)))==0) ? 1.0 : 0.0;
+		case Operator::factorof: {
+			int fb = int(floor(b));
+			if(KDE_ISLIKELY(fb!=0))
+				oper->setValue((int(floor(a)) % fb)==0);
 			else
 				*correct=new QString(i18n("Cannot calculate the factor on 0."));
-			
-			format=Cn::Boolean;
-			break;
+		}	break;
 		case Operator::min:
-			a= a < b? a : b;
+			oper->setValue(a < b? a : b);
 			break;
 		case Operator::max:
-			a= a > b? a : b;
+			oper->setValue(a > b? a : b);
 			break;
 		case Operator::gt:
-			a= a > b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a > b);
 			break;
 		case Operator::lt:
-			a= a < b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a < b);
 			break;
 		case Operator::eq:
-			a= a == b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a == b);
 			break;
 		case Operator::approx:
-			a= fabs(a-b)<0.001? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(fabs(a-b)<0.001);
 			break;
 		case Operator::neq:
-			a= a != b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a != b);
 			break;
 		case Operator::geq:
-			a= a >= b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a >= b);
 			break;
 		case Operator::leq:
-			a= a <= b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a <= b);
 			break;
 		case Operator::_and:
-			a= a && b? 1.0 : 0.0;
-			format=Cn::Boolean;
+			oper->setValue(a && b);
 			break;
 		case Operator::_or:
-			a= a || b? 1.0 : 0.0;
-			format = Cn::Boolean;
+			oper->setValue(a || b);
 			break;
 		case Operator::_xor:
-			a= (a || b) && !(a&&b)? 1.0 : 0.0;
-			format = Cn::Boolean;
+			oper->setValue((a || b) && !(a&&b));
 			break;
 		case Operator::implies:
-			a= (a || !b)? 0.0 : 1.0;
-			format = Cn::Boolean;
+			oper->setValue((a || !b));
 			break;
 		case Operator::gcd:  {
 			//code by michael cane aka kiko :)
@@ -136,8 +123,7 @@ Cn* Operations::reduceRealReal(enum Operator::OperatorType op, Cn *oper, const C
 				ia = ib;
 				ib = residu;
 			}
-			a=ia; b=ib;
-			format=Cn::Integer;
+			oper->setValue(ia);
 		}	break;
 		case Operator::lcm:
 			//code by michael cane aka kiko :)
@@ -152,144 +138,132 @@ Cn* Operations::reduceRealReal(enum Operator::OperatorType op, Cn *oper, const C
 					ib = residu;
 				}
 				ia=ic/ia;
-				a=ia; b=ib;
+				oper->setValue(ia);
 			}
-			format=Cn::Integer;
 			break;
 		case Operator::root:
-			a = b==2.0 ? sqrt(a) : pow(a, 1.0/b);
+			oper->setValue(b==2.0 ? sqrt(a) : pow(a, 1.0/b));
 			break;
 		default:
 			break;
 	}
-	oper->setValue(a);
-	oper->setFormat(format);
-	delete oper1;
 	return oper;
 }
 
 Cn* Operations::reduceUnaryReal(enum Operator::OperatorType op, Cn *val, QString** correct)
 {
-	double a=val->value();
-	Cn::ValueFormat format=val->format();
+	const double a=val->value();
 	
 	switch(op) {
 		case Operator::minus:
-			a = -a;
+			val->setValue(-a);
 			break;
 		case Operator::factorial: {
 			//Use gamma from math.h?
 			uint res=1;
-			for(; a>1.; a--) {
-				res*=floor(a);
+			for(int i=a; i>1.; i--) {
+				res*=floor(i);
 			}
-			a=res;
-			format=Cn::Integer;
+			val->setValue(res);
 		}	break;
 		case Operator::sin:
-			a=sin(a);
+			val->setValue(sin(a));
 			break;
 		case Operator::cos:
-			a=cos(a);
+			val->setValue(cos(a));
 			break;
 		case Operator::tan:
-			a=tan(a);
+			val->setValue(tan(a));
 			break;
 		case Operator::sec:
-			a=1./cos(a);
+			val->setValue(1./cos(a));
 			break;
 		case Operator::csc:
-			a=1./sin(a);
+			val->setValue(1./sin(a));
 			break;
 		case Operator::cot:
-			a=1./tan(a);
+			val->setValue(1./tan(a));
 			break;
 		case Operator::sinh:
-			a=sinh(a);
+			val->setValue(sinh(a));
 			break;
 		case Operator::cosh:
-			a=cosh(a);
+			val->setValue(cosh(a));
 			break;
 		case Operator::tanh:
-			a=tanh(a);
+			val->setValue(tanh(a));
 			break;
 		case Operator::sech:
-			a=1.0/cosh(a);
+			val->setValue(1.0/cosh(a));
 			break;
 		case Operator::csch:
-			a=1.0/sinh(a);
+			val->setValue(1.0/sinh(a));
 			break;
 		case Operator::coth:
-			a=cosh(a)/sinh(a);
+			val->setValue(cosh(a)/sinh(a));
 			break;
 		case Operator::arcsin:
-			a=asin(a);
+			val->setValue(asin(a));
 			break;
 		case Operator::arccos:
-			a=acos(a);
+			val->setValue(acos(a));
 			break;
 		case Operator::arctan:
-			a=atan(a);
+			val->setValue(atan(a));
 			break;
 		case Operator::arccot:
-			a=log(a+pow(a*a+1., 0.5));
+			val->setValue(log(a+pow(a*a+1., 0.5)));
 			break;
 		case Operator::arcsinh:
-			a=0.5*(log(1.+1./a)-log(1.-1./a));
+			val->setValue(0.5*(log(1.+1./a)-log(1.-1./a)));
 			break;
 		case Operator::arccosh:
-			a=log(a+sqrt(a-1.)*sqrt(a+1.));
+			val->setValue(log(a+sqrt(a-1.)*sqrt(a+1.)));
 			break;
 		case Operator::arccsc:
-			a=1/asin(a);
+			val->setValue(1/asin(a));
 			break;
 		case Operator::arccsch:
-			a=1/(0.5*(log(1.+1./a)-log(1.-1./a)));
+			val->setValue(1/(0.5*(log(1.+1./a)-log(1.-1./a))));
 			break;
 		case Operator::arcsec:
-			a=1/(acos(a));
+			val->setValue(1/(acos(a)));
 			break;
 		case Operator::arcsech:
-			a=1/(log(a+sqrt(a-1.)*sqrt(a+1.)));
+			val->setValue(1/(log(a+sqrt(a-1.)*sqrt(a+1.))));
 			break;
 		case Operator::arctanh:
-			a=atanh(a);
+			val->setValue(atanh(a));
 			break;
 		case Operator::exp:
-			a=exp(a);
+			val->setValue(exp(a));
 			break;
 		case Operator::ln:
-			a=log(a);
+			val->setValue(log(a));
 			break;
 		case Operator::log:
-			a=log10(a);
+			val->setValue(log10(a));
 			break;
 		case Operator::abs:
-			a= a>=0. ? a : -a;
+			val->setValue(a>=0. ? a : -a);
 			break;
 		//case Object::conjugate:
 		//case Object::arg:
 		//case Object::real:
 		//case Object::imaginary:
 		case Operator::floor:
-			a=floor(a);
-			format=Cn::Integer;
+			val->setValue(floor(a));
 			break;
 		case Operator::ceiling:
-			a=ceil(a);
-			format=Cn::Integer;
+			val->setValue(ceil(a));
 			break;
 		case Operator::_not:
-			a=!a;
-			format=Cn::Boolean;
+			val->setValue(!a);
 			break;
 		default:
 			*correct=new QString(i18n("Could not calculate a value %1", Operator(op).toString()));
 			break;
 	}
-	
-	val->setValue(a);
-	val->setFormat(format);
 	return val;
 }
 
@@ -298,26 +272,26 @@ Object * Operations::reduceRealVector(Operator::OperatorType op, Cn * oper, Vect
 	switch(op) {
 		case Operator::selector: {
 			int select=oper->intValue();
+			delete oper;
 			Object* ret=0;
 			if(select<1 || (select-1) >= v1->size()) {
 				*correct=new QString(i18n("Invalid index for a container"));
 				ret=new Cn(0.);
 			} else {
-				ret=v1->at(select-1);
-				v1->setAt(select-1, 0);
+				ret=v1->at(select-1)->copy();
 			}
-			delete oper;
-			delete v1;
 			return ret;
 		}	break;
-		default:
-			for(Vector::iterator it=v1->begin(); it!=v1->end(); ++it)
+		default: {
+			Vector *ret = v1->copy();
+			for(Vector::iterator it=ret->begin(); it!=ret->end(); ++it)
 			{
 				*it=reduce(op, new Cn(*oper), *it, correct);
 			}
 			
 			delete oper;
-			return v1;
+			return ret;
+		}
 	}
 }
 
@@ -327,13 +301,12 @@ Object * Operations::reduceVectorReal(Operator::OperatorType op, Vector * v1, Cn
 	{
 		*it=reduce(op, *it, new Cn(*oper), correct);
 	}
-	delete oper;
 	return v1;
 }
 
 Object * Operations::reduceVectorVector(Operator::OperatorType op, Vector * v1, Vector * v2, QString** correct)
 {
-	if(v1->size()!=v2->size()) {
+	if(v1->size()!=v2->size()) { //FIXME: unneeded?
 		*correct=new QString(i18n("Cannot operate on different sized vectors."));
 		return new Cn(0.);
 	}
@@ -341,13 +314,10 @@ Object * Operations::reduceVectorVector(Operator::OperatorType op, Vector * v1,
 	if(op==Operator::scalarproduct)
 		op=Operator::times;
 	Vector::iterator it2=v2->begin();
-	for(Vector::iterator it1=v1->begin(); it1!=v1->end(); ++it1)
+	for(Vector::iterator it1=v1->begin(); it1!=v1->end(); ++it1, ++it2)
 	{
 		*it1=reduce(op, *it1, *it2, correct);
-		
-		it2=v2->erase(it2);
 	}
-	delete v2;
 	return v1;
 }
 
@@ -374,9 +344,8 @@ Object* Operations::reduceListList(Operator::OperatorType op, List* l1, List* l2
 	switch(op) {
 		case Operator::_union: {
 			List::iterator itEnd=l2->end();
-			for(List::iterator it=l2->begin(); it!=itEnd;) {
-				l1->appendBranch(*it);
-				it=l2->erase(it);
+			for(List::iterator it=l2->begin(); it!=itEnd; ++it) {
+				l1->appendBranch((*it)->copy());
 			}
 			
 			ret=l1;
@@ -388,7 +357,6 @@ Object* Operations::reduceListList(Operator::OperatorType op, List* l1, List* l2
 			ret=new Cn(0.);
 			break;
 	}
-	delete l2;
 	return ret;
 }
 
@@ -422,7 +390,6 @@ Object* Operations::reduceRealList(Operator::OperatorType op, Cn* oper, List* v1
 				v1->setAt(select-1, 0);
 			}
 			delete oper;
-			delete v1;
 			return ret;
 		}	break;
 		default:
@@ -431,6 +398,56 @@ Object* Operations::reduceRealList(Operator::OperatorType op, Cn* oper, List* v1
 	return 0;
 }
 
+Object* Operations::reduceCustomCustom(Operator::OperatorType op, CustomObject* v1, CustomObject* v2, QString** )
+{
+	switch(op) {
+		case Operator::neq:
+			return new Cn(v1->value()!=v2->value());
+		case Operator::eq:
+			return new Cn(v1->value()==v2->value());
+		default:
+			break;
+	}
+	
+	Q_ASSERT(false && "not implemented, please report");
+	return 0;
+}
+
+Object* Operations::reduceMatrixMatrix(Operator::OperatorType op, Matrix* m1, Matrix* m2, QString** correct)
+{
+	Matrix::iterator it2=m2->begin();
+	for(Matrix::iterator it1=m1->begin(); it1!=m1->end(); ++it1)
+	{
+		*it1 = reduceVectorVector(op, static_cast<MatrixRow*>(*it1), static_cast<MatrixRow*>(*it2), correct);
+		
+		++m2;
+	}
+	return m1;
+}
+
+Object* Operations::reduceRealMatrix(Operator::OperatorType op, Cn* v, Matrix* m1, QString** correct)
+{
+	if(op==Operator::selector) {
+		Vector* ret;
+		int select=v->intValue();
+		if(select<1 || (select-1) >= m1->size()) {
+			*correct=new QString(i18n("Invalid index for a container"));
+			ret=new Vector(1);
+		} else {
+			MatrixRow* row = static_cast<MatrixRow*>(m1->values()[select-1]);
+			Vector* nv = new Vector(row->size());
+			for(Vector::iterator it=row->begin(); it!=row->end(); ++it) {
+				nv->appendBranch((*it));
+				*it = 0;
+			}
+			ret = nv;
+		}
+		delete v;
+		return ret;
+	}
+	return 0;
+}
+
 ExpressionType TypeTriplet(const ExpressionType& a,const ExpressionType& b,const ExpressionType& c) { return ExpressionType(ExpressionType::Lambda).addParameter(a).addParameter(b).addParameter(c); }
 
 //TODO: test that there's one output per input
@@ -445,6 +462,9 @@ QList<ExpressionType> Operations::infer(Operator::OperatorType op)
 			ret << TypeTriplet(ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Value), -1),
 							   ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Value), -1),
 							   ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Value), -1));
+			ret << TypeTriplet(ExpressionType(ExpressionType::Matrix, ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Value), -2), -1),
+							   ExpressionType(ExpressionType::Matrix, ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Value), -2), -1),
+							   ExpressionType(ExpressionType::Matrix, ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Value), -2), -1));
 			break;
 		case Operator::divide:
 			ret << TypeTriplet(ExpressionType(ExpressionType::Value), ExpressionType(ExpressionType::Value), ExpressionType(ExpressionType::Value));
@@ -510,6 +530,9 @@ QList<ExpressionType> Operations::infer(Operator::OperatorType op)
 			ret << TypeTriplet(ExpressionType(ExpressionType::Value),
 							   ExpressionType(ExpressionType::List, ExpressionType(ExpressionType::Any, 1)),
 							   ExpressionType(ExpressionType::Any, 1));
+			ret << TypeTriplet(ExpressionType(ExpressionType::Value),
+							   ExpressionType(ExpressionType::Matrix, ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Any, 1), -1), -2),
+							   ExpressionType(ExpressionType::Vector, ExpressionType(ExpressionType::Any, 1), -1));
 			break;
 		case Operator::_union:
 			ret << TypeTriplet(ExpressionType(ExpressionType::List, ExpressionType(ExpressionType::Any, 1)),
@@ -598,31 +621,18 @@ QList<ExpressionType> Operations::inferUnary(Operator::OperatorType op)
 	return ret;
 }
 
-Object* Operations::reduceCustomCustom(Operator::OperatorType op, CustomObject* v1, CustomObject* v2, QString** )
-{
-	switch(op) {
-		case Operator::neq:
-			return new Cn(v1->value()!=v2->value());
-		case Operator::eq:
-			return new Cn(v1->value()==v2->value());
-		default:
-			break;
-	}
-	
-	Q_ASSERT(false && "not implemented, please report");
-	return 0;
-}
-
 Operations::BinaryOp Operations::opsBinary[Object::custom+1][Object::custom+1] = {
-	{0,0,0,0,0,0,0,0,0},
-	{0, (Operations::BinaryOp) reduceRealReal, 0, (Operations::BinaryOp) reduceRealVector, (Operations::BinaryOp) reduceRealList,0,0,0},
-	{0,0,0,0,0,0,0,0,0},
-	{0, (Operations::BinaryOp) reduceVectorReal, 0, (Operations::BinaryOp) reduceVectorVector, 0,0,0,0},
-	{0, 0, 0,0, (Operations::BinaryOp) reduceListList, 0,0,0},
-	{0,0,0,0,0,0,0,0,0},
-	{0,0,0,0,0,0,0,0,0},
-	{0,0,0,0,0,0,0,0,0},
-	{0,0,0,0,0,0,0,0,(Operations::BinaryOp) reduceCustomCustom}
+	{0,0,0,0,0,0,0,0,0,0,0},
+	{0, (Operations::BinaryOp) reduceRealReal, 0, (Operations::BinaryOp) reduceRealVector, (Operations::BinaryOp) reduceRealList,0,0,0,(Operations::BinaryOp) reduceRealMatrix,0},
+	{0,0,0,0,0,0,0,0,0,0,0},
+	{0, (Operations::BinaryOp) reduceVectorReal, 0, (Operations::BinaryOp) reduceVectorVector, 0,0,0,0,0,0},
+	{0, 0, 0,0, (Operations::BinaryOp) reduceListList, 0,0,0,0,0},
+	{0,0,0,0,0,0,0,0,0,0,0},
+	{0,0,0,0,0,0,0,0,0,0,0},
+	{0,0,0,0,0,0,0,0,0,0,0},
+	{0,0,0,0,0,0,0,0, (Operations::BinaryOp) reduceMatrixMatrix,0,0},
+	{0,0,0,0,0,0,0,0,0,0,0},
+	{0,0,0,0,0,0,0,0,0,0,(Operations::BinaryOp) reduceCustomCustom}
 };
 
 Object * Operations::reduce(Operator::OperatorType op, Object * val1, Object * val2, QString** correct)
@@ -640,7 +650,7 @@ Operations::UnaryOp Operations::opsUnary[] = {
 	0, //variable
 	(Operations::UnaryOp) Operations::reduceUnaryVector,
 	(Operations::UnaryOp) Operations::reduceUnaryList,
-	0,0,0
+	0,0
 };
 
 Object * Operations::reduceUnary(Operator::OperatorType op, Object * val, QString** correct)
